@@ -1,7 +1,6 @@
 package openapi2conv
 
 import (
-	"encoding/json"
 	"errors"
 	"fmt"
 	"net/url"
@@ -14,15 +13,13 @@ import (
 
 // ToV3 converts an OpenAPIv2 spec to an OpenAPIv3 spec
 func ToV3(doc2 *openapi2.T) (*openapi3.T, error) {
-	stripNonCustomExtensions(doc2.Extensions)
-
 	doc3 := &openapi3.T{
-		OpenAPI:        "3.0.3",
-		Info:           &doc2.Info,
-		Components:     openapi3.Components{},
-		Tags:           doc2.Tags,
-		ExtensionProps: doc2.ExtensionProps,
-		ExternalDocs:   doc2.ExternalDocs,
+		OpenAPI:      "3.0.3",
+		Info:         &doc2.Info,
+		Components:   &openapi3.Components{},
+		Tags:         doc2.Tags,
+		Extensions:   stripNonExtensions(doc2.Extensions),
+		ExternalDocs: doc2.ExternalDocs,
 	}
 
 	if host := doc2.Host; host != "" {
@@ -53,7 +50,7 @@ func ToV3(doc2 *openapi2.T) (*openapi3.T, error) {
 		doc3.Components.Parameters = make(map[string]*openapi3.ParameterRef)
 		doc3.Components.RequestBodies = make(map[string]*openapi3.RequestBodyRef)
 		for k, parameter := range parameters {
-			v3Parameter, v3RequestBody, v3SchemaMap, err := ToV3Parameter(&doc3.Components, parameter, doc2.Consumes)
+			v3Parameter, v3RequestBody, v3SchemaMap, err := ToV3Parameter(doc3.Components, parameter, doc2.Consumes)
 			switch {
 			case err != nil:
 				return nil, err
@@ -72,7 +69,7 @@ func ToV3(doc2 *openapi2.T) (*openapi3.T, error) {
 	if paths := doc2.Paths; len(paths) != 0 {
 		doc3Paths := make(map[string]*openapi3.PathItem, len(paths))
 		for path, pathItem := range paths {
-			r, err := ToV3PathItem(doc2, &doc3.Components, pathItem, doc2.Consumes)
+			r, err := ToV3PathItem(doc2, doc3.Components, pathItem, doc2.Consumes)
 			if err != nil {
 				return nil, err
 			}
@@ -119,9 +116,8 @@ func ToV3(doc2 *openapi2.T) (*openapi3.T, error) {
 }
 
 func ToV3PathItem(doc2 *openapi2.T, components *openapi3.Components, pathItem *openapi2.PathItem, consumes []string) (*openapi3.PathItem, error) {
-	stripNonCustomExtensions(pathItem.Extensions)
 	doc3 := &openapi3.PathItem{
-		ExtensionProps: pathItem.ExtensionProps,
+		Extensions: stripNonExtensions(pathItem.Extensions),
 	}
 	for method, operation := range pathItem.Operations() {
 		doc3Operation, err := ToV3Operation(doc2, components, pathItem, operation, consumes)
@@ -150,14 +146,13 @@ func ToV3Operation(doc2 *openapi2.T, components *openapi3.Components, pathItem *
 	if operation == nil {
 		return nil, nil
 	}
-	stripNonCustomExtensions(operation.Extensions)
 	doc3 := &openapi3.Operation{
-		OperationID:    operation.OperationID,
-		Summary:        operation.Summary,
-		Description:    operation.Description,
-		Deprecated:     operation.Deprecated,
-		Tags:           operation.Tags,
-		ExtensionProps: operation.ExtensionProps,
+		OperationID: operation.OperationID,
+		Summary:     operation.Summary,
+		Description: operation.Description,
+		Deprecated:  operation.Deprecated,
+		Tags:        operation.Tags,
+		Extensions:  stripNonExtensions(operation.Extensions),
 	}
 	if v := operation.Security; v != nil {
 		doc3Security := ToV3SecurityRequirements(*v)
@@ -230,24 +225,22 @@ func ToV3Parameter(components *openapi3.Components, parameter *openapi2.Paramete
 		}
 		return &openapi3.ParameterRef{Ref: ToV3Ref(ref)}, nil, nil, nil
 	}
-	stripNonCustomExtensions(parameter.Extensions)
 
 	switch parameter.In {
 	case "body":
 		result := &openapi3.RequestBody{
-			Description:    parameter.Description,
-			Required:       parameter.Required,
-			ExtensionProps: parameter.ExtensionProps,
+			Description: parameter.Description,
+			Required:    parameter.Required,
+			Extensions:  stripNonExtensions(parameter.Extensions),
 		}
 		if parameter.Name != "" {
 			if result.Extensions == nil {
-				result.Extensions = make(map[string]interface{})
+				result.Extensions = make(map[string]interface{}, 1)
 			}
 			result.Extensions["x-originalParamName"] = parameter.Name
 		}
 
 		if schemaRef := parameter.Schema; schemaRef != nil {
-			// Assuming JSON
 			result.WithSchemaRef(ToV3SchemaRef(schemaRef), consumes)
 		}
 		return nil, &openapi3.RequestBodyRef{Value: result}, nil, nil
@@ -257,39 +250,37 @@ func ToV3Parameter(components *openapi3.Components, parameter *openapi2.Paramete
 		if typ == "file" {
 			format, typ = "binary", "string"
 		}
-		if parameter.ExtensionProps.Extensions == nil {
-			parameter.ExtensionProps.Extensions = make(map[string]interface{})
+		if parameter.Extensions == nil {
+			parameter.Extensions = make(map[string]interface{}, 1)
 		}
-		parameter.ExtensionProps.Extensions["x-formData-name"] = parameter.Name
+		parameter.Extensions["x-formData-name"] = parameter.Name
 		var required []string
 		if parameter.Required {
 			required = []string{parameter.Name}
 		}
-		schemaRef := &openapi3.SchemaRef{
-			Value: &openapi3.Schema{
-				Description:     parameter.Description,
-				Type:            typ,
-				ExtensionProps:  parameter.ExtensionProps,
-				Format:          format,
-				Enum:            parameter.Enum,
-				Min:             parameter.Minimum,
-				Max:             parameter.Maximum,
-				ExclusiveMin:    parameter.ExclusiveMin,
-				ExclusiveMax:    parameter.ExclusiveMax,
-				MinLength:       parameter.MinLength,
-				MaxLength:       parameter.MaxLength,
-				Default:         parameter.Default,
-				Items:           parameter.Items,
-				MinItems:        parameter.MinItems,
-				MaxItems:        parameter.MaxItems,
-				Pattern:         parameter.Pattern,
-				AllowEmptyValue: parameter.AllowEmptyValue,
-				UniqueItems:     parameter.UniqueItems,
-				MultipleOf:      parameter.MultipleOf,
-				Required:        required,
-			},
-		}
-		schemaRefMap := make(map[string]*openapi3.SchemaRef)
+		schemaRef := &openapi3.SchemaRef{Value: &openapi3.Schema{
+			Description:     parameter.Description,
+			Type:            typ,
+			Extensions:      stripNonExtensions(parameter.Extensions),
+			Format:          format,
+			Enum:            parameter.Enum,
+			Min:             parameter.Minimum,
+			Max:             parameter.Maximum,
+			ExclusiveMin:    parameter.ExclusiveMin,
+			ExclusiveMax:    parameter.ExclusiveMax,
+			MinLength:       parameter.MinLength,
+			MaxLength:       parameter.MaxLength,
+			Default:         parameter.Default,
+			Items:           parameter.Items,
+			MinItems:        parameter.MinItems,
+			MaxItems:        parameter.MaxItems,
+			Pattern:         parameter.Pattern,
+			AllowEmptyValue: parameter.AllowEmptyValue,
+			UniqueItems:     parameter.UniqueItems,
+			MultipleOf:      parameter.MultipleOf,
+			Required:        required,
+		}}
+		schemaRefMap := make(map[string]*openapi3.SchemaRef, 1)
 		schemaRefMap[parameter.Name] = schemaRef
 		return nil, nil, schemaRefMap, nil
 
@@ -304,11 +295,11 @@ func ToV3Parameter(components *openapi3.Components, parameter *openapi2.Paramete
 			schemaRefRef = schemaRef.Ref
 		}
 		result := &openapi3.Parameter{
-			In:             parameter.In,
-			Name:           parameter.Name,
-			Description:    parameter.Description,
-			Required:       required,
-			ExtensionProps: parameter.ExtensionProps,
+			In:          parameter.In,
+			Name:        parameter.Name,
+			Description: parameter.Description,
+			Required:    required,
+			Extensions:  stripNonExtensions(parameter.Extensions),
 			Schema: ToV3SchemaRef(&openapi3.SchemaRef{Value: &openapi3.Schema{
 				Type:            parameter.Type,
 				Format:          parameter.Format,
@@ -417,10 +408,9 @@ func ToV3Response(response *openapi2.Response, produces []string) (*openapi3.Res
 	if ref := response.Ref; ref != "" {
 		return &openapi3.ResponseRef{Ref: ToV3Ref(ref)}, nil
 	}
-	stripNonCustomExtensions(response.Extensions)
 	result := &openapi3.Response{
-		Description:    &response.Description,
-		ExtensionProps: response.ExtensionProps,
+		Description: &response.Description,
+		Extensions:  stripNonExtensions(response.Extensions),
 	}
 
 	// Default to "application/json" if "produces" is not specified.
@@ -479,19 +469,14 @@ func ToV3SchemaRef(schema *openapi3.SchemaRef) *openapi3.SchemaRef {
 	for k, v := range schema.Value.Properties {
 		schema.Value.Properties[k] = ToV3SchemaRef(v)
 	}
-	if v := schema.Value.AdditionalProperties; v != nil {
-		schema.Value.AdditionalProperties = ToV3SchemaRef(v)
+	if v := schema.Value.AdditionalProperties.Schema; v != nil {
+		schema.Value.AdditionalProperties.Schema = ToV3SchemaRef(v)
 	}
 	for i, v := range schema.Value.AllOf {
 		schema.Value.AllOf[i] = ToV3SchemaRef(v)
 	}
 	if val, ok := schema.Value.Extensions["x-nullable"]; ok {
-		var nullable bool
-
-		if err := json.Unmarshal(val.(json.RawMessage), &nullable); err == nil {
-			schema.Value.Nullable = nullable
-		}
-
+		schema.Value.Nullable, _ = val.(bool)
 		delete(schema.Value.Extensions, "x-nullable")
 	}
 
@@ -539,10 +524,9 @@ func ToV3SecurityScheme(securityScheme *openapi2.SecurityScheme) (*openapi3.Secu
 	if securityScheme == nil {
 		return nil, nil
 	}
-	stripNonCustomExtensions(securityScheme.Extensions)
 	result := &openapi3.SecurityScheme{
-		Description:    securityScheme.Description,
-		ExtensionProps: securityScheme.ExtensionProps,
+		Description: securityScheme.Description,
+		Extensions:  stripNonExtensions(securityScheme.Extensions),
 	}
 	switch securityScheme.Type {
 	case "basic":
@@ -586,21 +570,20 @@ func ToV3SecurityScheme(securityScheme *openapi2.SecurityScheme) (*openapi3.Secu
 
 // FromV3 converts an OpenAPIv3 spec to an OpenAPIv2 spec
 func FromV3(doc3 *openapi3.T) (*openapi2.T, error) {
-	doc2Responses, err := FromV3Responses(doc3.Components.Responses, &doc3.Components)
+	doc2Responses, err := FromV3Responses(doc3.Components.Responses, doc3.Components)
 	if err != nil {
 		return nil, err
 	}
-	stripNonCustomExtensions(doc3.Extensions)
-	schemas, parameters := FromV3Schemas(doc3.Components.Schemas, &doc3.Components)
+	schemas, parameters := FromV3Schemas(doc3.Components.Schemas, doc3.Components)
 	doc2 := &openapi2.T{
-		Swagger:        "2.0",
-		Info:           *doc3.Info,
-		Definitions:    schemas,
-		Parameters:     parameters,
-		Responses:      doc2Responses,
-		Tags:           doc3.Tags,
-		ExtensionProps: doc3.ExtensionProps,
-		ExternalDocs:   doc3.ExternalDocs,
+		Swagger:      "2.0",
+		Info:         *doc3.Info,
+		Definitions:  schemas,
+		Parameters:   parameters,
+		Responses:    doc2Responses,
+		Tags:         doc3.Tags,
+		Extensions:   stripNonExtensions(doc3.Extensions),
+		ExternalDocs: doc3.ExternalDocs,
 	}
 
 	isHTTPS := false
@@ -633,8 +616,7 @@ func FromV3(doc3 *openapi3.T) (*openapi2.T, error) {
 			continue
 		}
 		doc2.AddOperation(path, "GET", nil)
-		stripNonCustomExtensions(pathItem.Extensions)
-		addPathExtensions(doc2, path, pathItem.ExtensionProps)
+		addPathExtensions(doc2, path, stripNonExtensions(pathItem.Extensions))
 		for method, operation := range pathItem.Operations() {
 			if operation == nil {
 				continue
@@ -647,7 +629,7 @@ func FromV3(doc3 *openapi3.T) (*openapi2.T, error) {
 		}
 		params := openapi2.Parameters{}
 		for _, param := range pathItem.Parameters {
-			p, err := FromV3Parameter(param, &doc3.Components)
+			p, err := FromV3Parameter(param, doc3.Components)
 			if err != nil {
 				return nil, err
 			}
@@ -658,13 +640,13 @@ func FromV3(doc3 *openapi3.T) (*openapi2.T, error) {
 	}
 
 	for name, param := range doc3.Components.Parameters {
-		if doc2.Parameters[name], err = FromV3Parameter(param, &doc3.Components); err != nil {
+		if doc2.Parameters[name], err = FromV3Parameter(param, doc3.Components); err != nil {
 			return nil, err
 		}
 	}
 
 	for name, requestBodyRef := range doc3.Components.RequestBodies {
-		bodyOrRefParameters, formDataParameters, consumes, err := fromV3RequestBodies(name, requestBodyRef, &doc3.Components)
+		bodyOrRefParameters, formDataParameters, consumes, err := fromV3RequestBodies(name, requestBodyRef, doc3.Components)
 		if err != nil {
 			return nil, err
 		}
@@ -733,7 +715,7 @@ func fromV3RequestBodies(name string, requestBodyRef *openapi3.RequestBodyRef, c
 
 			paramName := name
 			if originalName, ok := requestBodyRef.Value.Extensions["x-originalParamName"]; ok {
-				json.Unmarshal(originalName.(json.RawMessage), &paramName)
+				paramName = originalName.(string)
 			}
 
 			var r *openapi2.Parameter
@@ -786,11 +768,11 @@ func FromV3SchemaRef(schema *openapi3.SchemaRef, components *openapi3.Components
 			required := false
 
 			value, _ := schema.Value.Extensions["x-formData-name"]
-			var originalName string
-			json.Unmarshal(value.(json.RawMessage), &originalName)
+			originalName, _ := value.(string)
 			for _, prop := range schema.Value.Required {
 				if originalName == prop {
 					required = true
+					break
 				}
 			}
 			return nil, &openapi2.Parameter{
@@ -812,7 +794,7 @@ func FromV3SchemaRef(schema *openapi3.SchemaRef, components *openapi3.Components
 				AllowEmptyValue: schema.Value.AllowEmptyValue,
 				UniqueItems:     schema.Value.UniqueItems,
 				MultipleOf:      schema.Value.MultipleOf,
-				ExtensionProps:  schema.Value.ExtensionProps,
+				Extensions:      stripNonExtensions(schema.Value.Extensions),
 				Required:        required,
 			}
 		}
@@ -828,8 +810,8 @@ func FromV3SchemaRef(schema *openapi3.SchemaRef, components *openapi3.Components
 	for _, key := range keys {
 		schema.Value.Properties[key], _ = FromV3SchemaRef(schema.Value.Properties[key], components)
 	}
-	if v := schema.Value.AdditionalProperties; v != nil {
-		schema.Value.AdditionalProperties, _ = FromV3SchemaRef(v, components)
+	if v := schema.Value.AdditionalProperties.Schema; v != nil {
+		schema.Value.AdditionalProperties.Schema, _ = FromV3SchemaRef(v, components)
 	}
 	for i, v := range schema.Value.AllOf {
 		schema.Value.AllOf[i], _ = FromV3SchemaRef(v, components)
@@ -854,9 +836,8 @@ func FromV3SecurityRequirements(requirements openapi3.SecurityRequirements) open
 }
 
 func FromV3PathItem(doc3 *openapi3.T, pathItem *openapi3.PathItem) (*openapi2.PathItem, error) {
-	stripNonCustomExtensions(pathItem.Extensions)
 	result := &openapi2.PathItem{
-		ExtensionProps: pathItem.ExtensionProps,
+		Extensions: stripNonExtensions(pathItem.Extensions),
 	}
 	for method, operation := range pathItem.Operations() {
 		r, err := FromV3Operation(doc3, operation)
@@ -866,7 +847,7 @@ func FromV3PathItem(doc3 *openapi3.T, pathItem *openapi3.PathItem) (*openapi2.Pa
 		result.SetOperation(method, r)
 	}
 	for _, parameter := range pathItem.Parameters {
-		p, err := FromV3Parameter(parameter, &doc3.Components)
+		p, err := FromV3Parameter(parameter, doc3.Components)
 		if err != nil {
 			return nil, err
 		}
@@ -910,23 +891,23 @@ func FromV3RequestBodyFormData(mediaType *openapi3.MediaType) openapi2.Parameter
 			}
 		}
 		parameter := &openapi2.Parameter{
-			Name:           propName,
-			Description:    val.Description,
-			Type:           typ,
-			In:             "formData",
-			ExtensionProps: val.ExtensionProps,
-			Enum:           val.Enum,
-			ExclusiveMin:   val.ExclusiveMin,
-			ExclusiveMax:   val.ExclusiveMax,
-			MinLength:      val.MinLength,
-			MaxLength:      val.MaxLength,
-			Default:        val.Default,
-			Items:          val.Items,
-			MinItems:       val.MinItems,
-			MaxItems:       val.MaxItems,
-			Maximum:        val.Max,
-			Minimum:        val.Min,
-			Pattern:        val.Pattern,
+			Name:         propName,
+			Description:  val.Description,
+			Type:         typ,
+			In:           "formData",
+			Extensions:   stripNonExtensions(val.Extensions),
+			Enum:         val.Enum,
+			ExclusiveMin: val.ExclusiveMin,
+			ExclusiveMax: val.ExclusiveMax,
+			MinLength:    val.MinLength,
+			MaxLength:    val.MaxLength,
+			Default:      val.Default,
+			Items:        val.Items,
+			MinItems:     val.MinItems,
+			MaxItems:     val.MaxItems,
+			Maximum:      val.Max,
+			Minimum:      val.Min,
+			Pattern:      val.Pattern,
 			// CollectionFormat: val.CollectionFormat,
 			// Format:          val.Format,
 			AllowEmptyValue: val.AllowEmptyValue,
@@ -943,21 +924,20 @@ func FromV3Operation(doc3 *openapi3.T, operation *openapi3.Operation) (*openapi2
 	if operation == nil {
 		return nil, nil
 	}
-	stripNonCustomExtensions(operation.Extensions)
 	result := &openapi2.Operation{
-		OperationID:    operation.OperationID,
-		Summary:        operation.Summary,
-		Description:    operation.Description,
-		Deprecated:     operation.Deprecated,
-		Tags:           operation.Tags,
-		ExtensionProps: operation.ExtensionProps,
+		OperationID: operation.OperationID,
+		Summary:     operation.Summary,
+		Description: operation.Description,
+		Deprecated:  operation.Deprecated,
+		Tags:        operation.Tags,
+		Extensions:  stripNonExtensions(operation.Extensions),
 	}
 	if v := operation.Security; v != nil {
 		resultSecurity := FromV3SecurityRequirements(*v)
 		result.Security = &resultSecurity
 	}
 	for _, parameter := range operation.Parameters {
-		r, err := FromV3Parameter(parameter, &doc3.Components)
+		r, err := FromV3Parameter(parameter, doc3.Components)
 		if err != nil {
 			return nil, err
 		}
@@ -970,7 +950,7 @@ func FromV3Operation(doc3 *openapi3.T, operation *openapi3.Operation) (*openapi2
 			return nil, errors.New("could not find a name for request body")
 		}
 
-		bodyOrRefParameters, formDataParameters, consumes, err := fromV3RequestBodies(name, v, &doc3.Components)
+		bodyOrRefParameters, formDataParameters, consumes, err := fromV3RequestBodies(name, v, doc3.Components)
 		if err != nil {
 			return nil, err
 		}
@@ -991,7 +971,7 @@ func FromV3Operation(doc3 *openapi3.T, operation *openapi3.Operation) (*openapi2
 	sort.Sort(result.Parameters)
 
 	if responses := operation.Responses; responses != nil {
-		resultResponses, err := FromV3Responses(responses, &doc3.Components)
+		resultResponses, err := FromV3Responses(responses, doc3.Components)
 		if err != nil {
 			return nil, err
 		}
@@ -1003,13 +983,12 @@ func FromV3Operation(doc3 *openapi3.T, operation *openapi3.Operation) (*openapi2
 func FromV3RequestBody(name string, requestBodyRef *openapi3.RequestBodyRef, mediaType *openapi3.MediaType, components *openapi3.Components) (*openapi2.Parameter, error) {
 	requestBody := requestBodyRef.Value
 
-	stripNonCustomExtensions(requestBody.Extensions)
 	result := &openapi2.Parameter{
-		In:             "body",
-		Name:           name,
-		Description:    requestBody.Description,
-		Required:       requestBody.Required,
-		ExtensionProps: requestBody.ExtensionProps,
+		In:          "body",
+		Name:        name,
+		Description: requestBody.Description,
+		Required:    requestBody.Required,
+		Extensions:  stripNonExtensions(requestBody.Extensions),
 	}
 
 	if mediaType != nil {
@@ -1026,13 +1005,12 @@ func FromV3Parameter(ref *openapi3.ParameterRef, components *openapi3.Components
 	if parameter == nil {
 		return nil, nil
 	}
-	stripNonCustomExtensions(parameter.Extensions)
 	result := &openapi2.Parameter{
-		Description:    parameter.Description,
-		In:             parameter.In,
-		Name:           parameter.Name,
-		Required:       parameter.Required,
-		ExtensionProps: parameter.ExtensionProps,
+		Description: parameter.Description,
+		In:          parameter.In,
+		Name:        parameter.Name,
+		Required:    parameter.Required,
+		Extensions:  stripNonExtensions(parameter.Extensions),
 	}
 	if schemaRef := parameter.Schema; schemaRef != nil {
 		schemaRef, _ = FromV3SchemaRef(schemaRef, components)
@@ -1088,10 +1066,9 @@ func FromV3Response(ref *openapi3.ResponseRef, components *openapi3.Components)
 	if desc := response.Description; desc != nil {
 		description = *desc
 	}
-	stripNonCustomExtensions(response.Extensions)
 	result := &openapi2.Response{
-		Description:    description,
-		ExtensionProps: response.ExtensionProps,
+		Description: description,
+		Extensions:  stripNonExtensions(response.Extensions),
 	}
 	if content := response.Content; content != nil {
 		if ct := content["application/json"]; ct != nil {
@@ -1127,11 +1104,10 @@ func FromV3SecurityScheme(doc3 *openapi3.T, ref *openapi3.SecuritySchemeRef) (*o
 	if securityScheme == nil {
 		return nil, nil
 	}
-	stripNonCustomExtensions(securityScheme.Extensions)
 	result := &openapi2.SecurityScheme{
-		Ref:            FromV3Ref(ref.Ref),
-		Description:    securityScheme.Description,
-		ExtensionProps: securityScheme.ExtensionProps,
+		Ref:         FromV3Ref(ref.Ref),
+		Description: securityScheme.Description,
+		Extensions:  stripNonExtensions(securityScheme.Extensions),
 	}
 	switch securityScheme.Type {
 	case "http":
@@ -1195,24 +1171,24 @@ var attemptedBodyParameterNames = []string{
 	"requestBody",
 }
 
-func stripNonCustomExtensions(extensions map[string]interface{}) {
+// stripNonExtensions removes invalid extensions: those not prefixed by "x-" and returns them
+func stripNonExtensions(extensions map[string]interface{}) map[string]interface{} {
 	for extName := range extensions {
 		if !strings.HasPrefix(extName, "x-") {
 			delete(extensions, extName)
 		}
 	}
+	return extensions
 }
 
-func addPathExtensions(doc2 *openapi2.T, path string, extensionProps openapi3.ExtensionProps) {
-	paths := doc2.Paths
-	if paths == nil {
-		paths = make(map[string]*openapi2.PathItem, 8)
-		doc2.Paths = paths
+func addPathExtensions(doc2 *openapi2.T, path string, extensions map[string]interface{}) {
+	if doc2.Paths == nil {
+		doc2.Paths = make(map[string]*openapi2.PathItem)
 	}
-	pathItem := paths[path]
+	pathItem := doc2.Paths[path]
 	if pathItem == nil {
 		pathItem = &openapi2.PathItem{}
-		paths[path] = pathItem
+		doc2.Paths[path] = pathItem
 	}
-	pathItem.ExtensionProps = extensionProps
+	pathItem.Extensions = extensions
 }
