@@ -221,8 +221,6 @@ func TestExamplesSchemaValidation(t *testing.T) {
 			t.Parallel()
 			for _, tc := range testCases {
 				t.Run(tc.name, func(t *testing.T) {
-					loader := NewLoader()
-
 					spec := bytes.Buffer{}
 					spec.WriteString(`
 openapi: 3.0.3
@@ -243,6 +241,7 @@ paths:
 					spec.WriteString(tc.parametersExample)
 					spec.WriteString(`
       requestBody:
+        required: true
         content:
           application/json:
             schema:
@@ -251,7 +250,6 @@ paths:
 					spec.WriteString(tc.mediaTypeRequestExample)
 					spec.WriteString(`
         description: Created user object
-        required: true
       responses:
         '204':
           description: "success"
@@ -264,11 +262,12 @@ paths:
   /readWriteOnly:
     post:
       requestBody:
+        required: true
         content:
           application/json:
             schema:
               $ref: "#/components/schemas/ReadWriteOnlyData"
-            required: true`)
+`)
 					spec.WriteString(tc.readWriteOnlyMediaTypeRequestExample)
 					spec.WriteString(`
       responses:
@@ -339,13 +338,14 @@ components:
 `)
 					spec.WriteString(tc.componentExamples)
 
+					loader := NewLoader()
 					doc, err := loader.LoadFromData(spec.Bytes())
 					require.NoError(t, err)
 
 					if testOption.disableExamplesValidation {
 						err = doc.Validate(loader.Context, DisableExamplesValidation())
 					} else {
-						err = doc.Validate(loader.Context)
+						err = doc.Validate(loader.Context, EnableExamplesValidation())
 					}
 
 					if tc.errContains != "" && !testOption.disableExamplesValidation {
@@ -436,8 +436,6 @@ func TestExampleObjectValidation(t *testing.T) {
 			t.Parallel()
 			for _, tc := range testCases {
 				t.Run(tc.name, func(t *testing.T) {
-					loader := NewLoader()
-
 					spec := bytes.Buffer{}
 					spec.WriteString(`
 openapi: 3.0.3
@@ -506,6 +504,7 @@ components:
 `)
 					spec.WriteString(tc.componentExamples)
 
+					loader := NewLoader()
 					doc, err := loader.LoadFromData(spec.Bytes())
 					require.NoError(t, err)
 
