@@ -44,6 +44,7 @@ type Loader struct {
 
 	visitedDocuments map[string]*T
 
+	visitedCallback       map[*Callback]struct{}
 	visitedExample        map[*Example]struct{}
 	visitedHeader         map[*Header]struct{}
 	visitedLink           map[*Link]struct{}
@@ -115,7 +116,7 @@ func (loader *Loader) loadSingleElementFromURI(ref string, rootPath *url.URL, el
 	if err != nil {
 		return nil, err
 	}
-	if err := yaml.Unmarshal(data, element); err != nil {
+	if err := unmarshal(data, element); err != nil {
 		return nil, err
 	}
 
@@ -133,7 +134,7 @@ func (loader *Loader) readURL(location *url.URL) ([]byte, error) {
 func (loader *Loader) LoadFromData(data []byte) (*T, error) {
 	loader.resetVisitedPathItemRefs()
 	doc := &T{}
-	if err := yaml.Unmarshal(data, doc); err != nil {
+	if err := unmarshal(data, doc); err != nil {
 		return nil, err
 	}
 	if err := loader.ResolveRefsIn(doc, nil); err != nil {
@@ -162,7 +163,7 @@ func (loader *Loader) loadFromDataWithPathInternal(data []byte, location *url.UR
 	doc := &T{}
 	loader.visitedDocuments[uri] = doc
 
-	if err := yaml.Unmarshal(data, doc); err != nil {
+	if err := unmarshal(data, doc); err != nil {
 		return nil, err
 	}
 	if err := loader.ResolveRefsIn(doc, location); err != nil {
@@ -172,6 +173,15 @@ func (loader *Loader) loadFromDataWithPathInternal(data []byte, location *url.UR
 	return doc, nil
 }
 
+func unmarshal(data []byte, v interface{}) error {
+	// See https://github.com/getkin/kin-openapi/issues/680
+	if err := json.Unmarshal(data, v); err != nil {
+		// UnmarshalStrict(data, v) TODO: investigate how ymlv3 handles duplicate map keys
+		return yaml.Unmarshal(data, v)
+	}
+	return nil
+}
+
 // ResolveRefsIn expands references if for instance spec was just unmarshalled
 func (loader *Loader) ResolveRefsIn(doc *T, location *url.URL) (err error) {
 	if loader.Context == nil {
@@ -182,63 +192,63 @@ func (loader *Loader) ResolveRefsIn(doc *T, location *url.URL) (err error) {
 		loader.resetVisitedPathItemRefs()
 	}
 
-	// Visit all components
-	components := doc.Components
-	for _, component := range components.Headers {
-		if err = loader.resolveHeaderRef(doc, component, location); err != nil {
-			return
+	if components := doc.Components; components != nil {
+		for _, component := range components.Headers {
+			if err = loader.resolveHeaderRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
-	for _, component := range components.Parameters {
-		if err = loader.resolveParameterRef(doc, component, location); err != nil {
-			return
+		for _, component := range components.Parameters {
+			if err = loader.resolveParameterRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
-	for _, component := range components.RequestBodies {
-		if err = loader.resolveRequestBodyRef(doc, component, location); err != nil {
-			return
+		for _, component := range components.RequestBodies {
+			if err = loader.resolveRequestBodyRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
-	for _, component := range components.Responses {
-		if err = loader.resolveResponseRef(doc, component, location); err != nil {
-			return
+		for _, component := range components.Responses {
+			if err = loader.resolveResponseRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
-	for _, component := range components.Schemas {
-		if err = loader.resolveSchemaRef(doc, component, location, []string{}); err != nil {
-			return
+		for _, component := range components.Schemas {
+			if err = loader.resolveSchemaRef(doc, component, location, []string{}); err != nil {
+				return
+			}
 		}
-	}
-	for _, component := range components.SecuritySchemes {
-		if err = loader.resolveSecuritySchemeRef(doc, component, location); err != nil {
-			return
+		for _, component := range components.SecuritySchemes {
+			if err = loader.resolveSecuritySchemeRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
 
-	examples := make([]string, 0, len(components.Examples))
-	for name := range components.Examples {
-		examples = append(examples, name)
-	}
-	sort.Strings(examples)
-	for _, name := range examples {
-		component := components.Examples[name]
-		if err = loader.resolveExampleRef(doc, component, location); err != nil {
-			return
+		examples := make([]string, 0, len(components.Examples))
+		for name := range components.Examples {
+			examples = append(examples, name)
+		}
+		sort.Strings(examples)
+		for _, name := range examples {
+			component := components.Examples[name]
+			if err = loader.resolveExampleRef(doc, component, location); err != nil {
+				return
+			}
 		}
-	}
 
-	for _, component := range components.Callbacks {
-		if err = loader.resolveCallbackRef(doc, component, location); err != nil {
-			return
+		for _, component := range components.Callbacks {
+			if err = loader.resolveCallbackRef(doc, component, location); err != nil {
+				return
+			}
 		}
 	}
 
 	// Visit all operations
-	for entrypoint, pathItem := range doc.Paths {
+	for _, pathItem := range doc.Paths {
 		if pathItem == nil {
 			continue
 		}
-		if err = loader.resolvePathItemRef(doc, entrypoint, pathItem, location); err != nil {
+		if err = loader.resolvePathItemRef(doc, pathItem, location); err != nil {
 			return
 		}
 	}
@@ -273,26 +283,22 @@ func isSingleRefElement(ref string) bool {
 	return !strings.Contains(ref, "#")
 }
 
-func (loader *Loader) resolveComponent(
-	doc *T,
-	ref string,
-	path *url.URL,
-	resolved interface{},
-) (
+func (loader *Loader) resolveComponent(doc *T, ref string, path *url.URL, resolved interface{}) (
+	componentDoc *T,
 	componentPath *url.URL,
 	err error,
 ) {
-	if doc, ref, componentPath, err = loader.resolveRef(doc, ref, path); err != nil {
-		return nil, err
+	if componentDoc, ref, componentPath, err = loader.resolveRef(doc, ref, path); err != nil {
+		return nil, nil, err
 	}
 
 	parsedURL, err := url.Parse(ref)
 	if err != nil {
-		return nil, fmt.Errorf("cannot parse reference: %q: %v", ref, parsedURL)
+		return nil, nil, fmt.Errorf("cannot parse reference: %q: %v", ref, parsedURL)
 	}
 	fragment := parsedURL.Fragment
 	if !strings.HasPrefix(fragment, "/") {
-		return nil, fmt.Errorf("expected fragment prefix '#/' in URI %q", ref)
+		return nil, nil, fmt.Errorf("expected fragment prefix '#/' in URI %q", ref)
 	}
 
 	drill := func(cursor interface{}) (interface{}, error) {
@@ -310,20 +316,20 @@ func (loader *Loader) resolveComponent(
 		return cursor, nil
 	}
 	var cursor interface{}
-	if cursor, err = drill(doc); err != nil {
+	if cursor, err = drill(componentDoc); err != nil {
 		if path == nil {
-			return nil, err
+			return nil, nil, err
 		}
 		var err2 error
 		data, err2 := loader.readURL(path)
 		if err2 != nil {
-			return nil, err
+			return nil, nil, err
 		}
-		if err2 = yaml.Unmarshal(data, &cursor); err2 != nil {
-			return nil, err
+		if err2 = unmarshal(data, &cursor); err2 != nil {
+			return nil, nil, err
 		}
 		if cursor, err2 = drill(cursor); err2 != nil || cursor == nil {
-			return nil, err
+			return nil, nil, err
 		}
 		err = nil
 	}
@@ -331,7 +337,7 @@ func (loader *Loader) resolveComponent(
 	switch {
 	case reflect.TypeOf(cursor) == reflect.TypeOf(resolved):
 		reflect.ValueOf(resolved).Elem().Set(reflect.ValueOf(cursor).Elem())
-		return componentPath, nil
+		return componentDoc, componentPath, nil
 
 	case reflect.TypeOf(cursor) == reflect.TypeOf(map[string]interface{}{}):
 		codec := func(got, expect interface{}) error {
@@ -345,22 +351,51 @@ func (loader *Loader) resolveComponent(
 			return nil
 		}
 		if err := codec(cursor, resolved); err != nil {
-			return nil, fmt.Errorf("bad data in %q", ref)
+			return nil, nil, fmt.Errorf("bad data in %q (expecting %s)", ref, readableType(resolved))
 		}
-		return componentPath, nil
+		return componentDoc, componentPath, nil
 
 	default:
-		return nil, fmt.Errorf("bad data in %q", ref)
+		return nil, nil, fmt.Errorf("bad data in %q (expecting %s)", ref, readableType(resolved))
+	}
+}
+
+func readableType(x interface{}) string {
+	switch x.(type) {
+	case *Callback:
+		return "callback object"
+	case *CallbackRef:
+		return "ref to callback object"
+	case *ExampleRef:
+		return "ref to example object"
+	case *HeaderRef:
+		return "ref to header object"
+	case *LinkRef:
+		return "ref to link object"
+	case *ParameterRef:
+		return "ref to parameter object"
+	case *PathItem:
+		return "pathItem object"
+	case *RequestBodyRef:
+		return "ref to requestBody object"
+	case *ResponseRef:
+		return "ref to response object"
+	case *SchemaRef:
+		return "ref to schema object"
+	case *SecuritySchemeRef:
+		return "ref to securityScheme object"
+	default:
+		panic(fmt.Sprintf("unreachable %T", x))
 	}
 }
 
 func drillIntoField(cursor interface{}, fieldName string) (interface{}, error) {
 	// Special case due to multijson
 	if s, ok := cursor.(*SchemaRef); ok && fieldName == "additionalProperties" {
-		if ap := s.Value.AdditionalPropertiesAllowed; ap != nil {
+		if ap := s.Value.AdditionalProperties.Has; ap != nil {
 			return *ap, nil
 		}
-		return s.Value.AdditionalProperties, nil
+		return s.Value.AdditionalProperties.Schema, nil
 	}
 
 	switch val := reflect.Indirect(reflect.ValueOf(cursor)); val.Kind() {
@@ -386,14 +421,7 @@ func drillIntoField(cursor interface{}, fieldName string) (interface{}, error) {
 		hasFields := false
 		for i := 0; i < val.NumField(); i++ {
 			hasFields = true
-			field := val.Type().Field(i)
-			tagValue := field.Tag.Get("yaml")
-			yamlKey := strings.Split(tagValue, ",")[0]
-			if yamlKey == "-" {
-				tagValue := field.Tag.Get("multijson")
-				yamlKey = strings.Split(tagValue, ",")[0]
-			}
-			if yamlKey == fieldName {
+			if fieldName == strings.Split(val.Type().Field(i).Tag.Get("yaml"), ",")[0] {
 				return val.Field(i).Interface(), nil
 			}
 		}
@@ -403,14 +431,10 @@ func drillIntoField(cursor interface{}, fieldName string) (interface{}, error) {
 			return drillIntoField(val.FieldByName("Value").Interface(), fieldName)
 		}
 		if hasFields {
-			if ff := val.Type().Field(0); ff.PkgPath == "" && ff.Name == "ExtensionProps" {
-				extensions := val.Field(0).Interface().(ExtensionProps).Extensions
+			if ff := val.Type().Field(0); ff.PkgPath == "" && ff.Name == "Extensions" {
+				extensions := val.Field(0).Interface().(map[string]interface{})
 				if enc, ok := extensions[fieldName]; ok {
-					var dec interface{}
-					if err := json.Unmarshal(enc.(json.RawMessage), &dec); err != nil {
-						return nil, err
-					}
-					return dec, nil
+					return enc, nil
 				}
 			}
 		}
@@ -421,18 +445,6 @@ func drillIntoField(cursor interface{}, fieldName string) (interface{}, error) {
 	}
 }
 
-func (loader *Loader) documentPathForRecursiveRef(current *url.URL, resolvedRef string) *url.URL {
-	if loader.rootDir == "" {
-		return current
-	}
-
-	if resolvedRef == "" {
-		return &url.URL{Path: loader.rootLocation}
-	}
-
-	return &url.URL{Path: path.Join(loader.rootDir, resolvedRef)}
-}
-
 func (loader *Loader) resolveRef(doc *T, ref string, path *url.URL) (*T, string, *url.URL, error) {
 	if ref != "" && ref[0] == '#' {
 		return doc, ref, path, nil
@@ -484,7 +496,7 @@ func (loader *Loader) resolveHeaderRef(doc *T, component *HeaderRef, documentPat
 			component.Value = &header
 		} else {
 			var resolved HeaderRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -492,7 +504,6 @@ func (loader *Loader) resolveHeaderRef(doc *T, component *HeaderRef, documentPat
 				return err
 			}
 			component.Value = resolved.Value
-			documentPath = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	value := component.Value
@@ -532,7 +543,7 @@ func (loader *Loader) resolveParameterRef(doc *T, component *ParameterRef, docum
 			component.Value = &param
 		} else {
 			var resolved ParameterRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -540,7 +551,6 @@ func (loader *Loader) resolveParameterRef(doc *T, component *ParameterRef, docum
 				return err
 			}
 			component.Value = resolved.Value
-			documentPath = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	value := component.Value
@@ -589,7 +599,7 @@ func (loader *Loader) resolveRequestBodyRef(doc *T, component *RequestBodyRef, d
 			component.Value = &requestBody
 		} else {
 			var resolved RequestBodyRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -597,7 +607,6 @@ func (loader *Loader) resolveRequestBodyRef(doc *T, component *RequestBodyRef, d
 				return err
 			}
 			component.Value = resolved.Value
-			documentPath = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	value := component.Value
@@ -651,7 +660,7 @@ func (loader *Loader) resolveResponseRef(doc *T, component *ResponseRef, documen
 			component.Value = &resp
 		} else {
 			var resolved ResponseRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -659,7 +668,6 @@ func (loader *Loader) resolveResponseRef(doc *T, component *ResponseRef, documen
 				return err
 			}
 			component.Value = resolved.Value
-			documentPath = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	value := component.Value
@@ -734,7 +742,7 @@ func (loader *Loader) resolveSchemaRef(doc *T, component *SchemaRef, documentPat
 			visited = append(visited, ref)
 
 			var resolved SchemaRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -742,11 +750,6 @@ func (loader *Loader) resolveSchemaRef(doc *T, component *SchemaRef, documentPat
 				return err
 			}
 			component.Value = resolved.Value
-			foundPath, rerr := loader.getResolvedRefPath(ref, &resolved, documentPath, componentPath)
-			if rerr != nil {
-				return fmt.Errorf("failed to resolve file from reference %q: %w", ref, rerr)
-			}
-			documentPath = loader.documentPathForRecursiveRef(documentPath, foundPath)
 		}
 		if loader.visitedSchema == nil {
 			loader.visitedSchema = make(map[*Schema]struct{})
@@ -769,7 +772,7 @@ func (loader *Loader) resolveSchemaRef(doc *T, component *SchemaRef, documentPat
 			return err
 		}
 	}
-	if v := value.AdditionalProperties; v != nil {
+	if v := value.AdditionalProperties.Schema; v != nil {
 		if err := loader.resolveSchemaRef(doc, v, documentPath, visited); err != nil {
 			return err
 		}
@@ -797,30 +800,6 @@ func (loader *Loader) resolveSchemaRef(doc *T, component *SchemaRef, documentPat
 	return nil
 }
 
-func (loader *Loader) getResolvedRefPath(ref string, resolved *SchemaRef, cur, found *url.URL) (string, error) {
-	if referencedFilename := strings.Split(ref, "#")[0]; referencedFilename == "" {
-		if cur != nil {
-			if loader.rootDir != "" && strings.HasPrefix(cur.Path, loader.rootDir) {
-				return cur.Path[len(loader.rootDir)+1:], nil
-			}
-
-			return path.Base(cur.Path), nil
-		}
-		return "", nil
-	}
-	// ref. to external file
-	if resolved.Ref != "" {
-		return resolved.Ref, nil
-	}
-
-	if loader.rootDir == "" {
-		return found.Path, nil
-	}
-
-	// found dest spec. file
-	return filepath.Rel(loader.rootDir, found.Path)
-}
-
 func (loader *Loader) resolveSecuritySchemeRef(doc *T, component *SecuritySchemeRef, documentPath *url.URL) (err error) {
 	if component != nil && component.Value != nil {
 		if loader.visitedSecurityScheme == nil {
@@ -844,7 +823,7 @@ func (loader *Loader) resolveSecuritySchemeRef(doc *T, component *SecurityScheme
 			component.Value = &scheme
 		} else {
 			var resolved SecuritySchemeRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -852,7 +831,6 @@ func (loader *Loader) resolveSecuritySchemeRef(doc *T, component *SecurityScheme
 				return err
 			}
 			component.Value = resolved.Value
-			_ = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	return nil
@@ -881,7 +859,7 @@ func (loader *Loader) resolveExampleRef(doc *T, component *ExampleRef, documentP
 			component.Value = &example
 		} else {
 			var resolved ExampleRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -889,13 +867,22 @@ func (loader *Loader) resolveExampleRef(doc *T, component *ExampleRef, documentP
 				return err
 			}
 			component.Value = resolved.Value
-			_ = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	return nil
 }
 
 func (loader *Loader) resolveCallbackRef(doc *T, component *CallbackRef, documentPath *url.URL) (err error) {
+	if component != nil && component.Value != nil {
+		if loader.visitedCallback == nil {
+			loader.visitedCallback = make(map[*Callback]struct{})
+		}
+		if _, ok := loader.visitedCallback[component.Value]; ok {
+			return nil
+		}
+		loader.visitedCallback[component.Value] = struct{}{}
+	}
+
 	if component == nil {
 		return errors.New("invalid callback: value MUST be an object")
 	}
@@ -908,15 +895,14 @@ func (loader *Loader) resolveCallbackRef(doc *T, component *CallbackRef, documen
 			component.Value = &resolved
 		} else {
 			var resolved CallbackRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
-			if err := loader.resolveCallbackRef(doc, &resolved, componentPath); err != nil {
+			if err = loader.resolveCallbackRef(doc, &resolved, componentPath); err != nil {
 				return err
 			}
 			component.Value = resolved.Value
-			documentPath = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	value := component.Value
@@ -924,59 +910,8 @@ func (loader *Loader) resolveCallbackRef(doc *T, component *CallbackRef, documen
 		return nil
 	}
 
-	for entrypoint, pathItem := range *value {
-		entrypoint, pathItem := entrypoint, pathItem
-		err = func() (err error) {
-			key := "-"
-			if documentPath != nil {
-				key = documentPath.EscapedPath()
-			}
-			key += entrypoint
-			if _, ok := loader.visitedPathItemRefs[key]; ok {
-				return nil
-			}
-			loader.visitedPathItemRefs[key] = struct{}{}
-
-			if pathItem == nil {
-				return errors.New("invalid path item: value MUST be an object")
-			}
-			ref := pathItem.Ref
-			if ref != "" {
-				if isSingleRefElement(ref) {
-					var p PathItem
-					if documentPath, err = loader.loadSingleElementFromURI(ref, documentPath, &p); err != nil {
-						return err
-					}
-					*pathItem = p
-				} else {
-					if doc, ref, documentPath, err = loader.resolveRef(doc, ref, documentPath); err != nil {
-						return
-					}
-
-					rest := strings.TrimPrefix(ref, "#/components/callbacks/")
-					if rest == ref {
-						return fmt.Errorf(`expected prefix "#/components/callbacks/" in URI %q`, ref)
-					}
-					id := unescapeRefString(rest)
-
-					definitions := doc.Components.Callbacks
-					if definitions == nil {
-						return failedToResolveRefFragmentPart(ref, "callbacks")
-					}
-					resolved := definitions[id]
-					if resolved == nil {
-						return failedToResolveRefFragmentPart(ref, id)
-					}
-
-					for _, p := range *resolved.Value {
-						*pathItem = *p
-						break
-					}
-				}
-			}
-			return loader.resolvePathItemRefContinued(doc, pathItem, documentPath)
-		}()
-		if err != nil {
+	for _, pathItem := range *value {
+		if err = loader.resolvePathItemRef(doc, pathItem, documentPath); err != nil {
 			return err
 		}
 	}
@@ -1006,7 +941,7 @@ func (loader *Loader) resolveLinkRef(doc *T, component *LinkRef, documentPath *u
 			component.Value = &link
 		} else {
 			var resolved LinkRef
-			componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
+			doc, componentPath, err := loader.resolveComponent(doc, ref, documentPath, &resolved)
 			if err != nil {
 				return err
 			}
@@ -1014,28 +949,32 @@ func (loader *Loader) resolveLinkRef(doc *T, component *LinkRef, documentPath *u
 				return err
 			}
 			component.Value = resolved.Value
-			_ = loader.documentPathForRecursiveRef(documentPath, resolved.Ref)
 		}
 	}
 	return nil
 }
 
-func (loader *Loader) resolvePathItemRef(doc *T, entrypoint string, pathItem *PathItem, documentPath *url.URL) (err error) {
-	key := "_"
-	if documentPath != nil {
-		key = documentPath.EscapedPath()
-	}
-	key += entrypoint
-	if _, ok := loader.visitedPathItemRefs[key]; ok {
-		return nil
-	}
-	loader.visitedPathItemRefs[key] = struct{}{}
-
+func (loader *Loader) resolvePathItemRef(doc *T, pathItem *PathItem, documentPath *url.URL) (err error) {
 	if pathItem == nil {
 		return errors.New("invalid path item: value MUST be an object")
 	}
 	ref := pathItem.Ref
 	if ref != "" {
+		if pathItem.Summary != "" ||
+			pathItem.Description != "" ||
+			pathItem.Connect != nil ||
+			pathItem.Delete != nil ||
+			pathItem.Get != nil ||
+			pathItem.Head != nil ||
+			pathItem.Options != nil ||
+			pathItem.Patch != nil ||
+			pathItem.Post != nil ||
+			pathItem.Put != nil ||
+			pathItem.Trace != nil ||
+			len(pathItem.Servers) != 0 ||
+			len(pathItem.Parameters) != 0 {
+			return nil
+		}
 		if isSingleRefElement(ref) {
 			var p PathItem
 			if documentPath, err = loader.loadSingleElementFromURI(ref, documentPath, &p); err != nil {
@@ -1043,32 +982,15 @@ func (loader *Loader) resolvePathItemRef(doc *T, entrypoint string, pathItem *Pa
 			}
 			*pathItem = p
 		} else {
-			if doc, ref, documentPath, err = loader.resolveRef(doc, ref, documentPath); err != nil {
-				return
+			var resolved PathItem
+			if doc, documentPath, err = loader.resolveComponent(doc, ref, documentPath, &resolved); err != nil {
+				return err
 			}
-
-			rest := strings.TrimPrefix(ref, "#/paths/")
-			if rest == ref {
-				return fmt.Errorf(`expected prefix "#/paths/" in URI %q`, ref)
-			}
-			id := unescapeRefString(rest)
-
-			definitions := doc.Paths
-			if definitions == nil {
-				return failedToResolveRefFragmentPart(ref, "paths")
-			}
-			resolved := definitions[id]
-			if resolved == nil {
-				return failedToResolveRefFragmentPart(ref, id)
-			}
-
-			*pathItem = *resolved
+			*pathItem = resolved
 		}
+		pathItem.Ref = ref
 	}
-	return loader.resolvePathItemRefContinued(doc, pathItem, documentPath)
-}
 
-func (loader *Loader) resolvePathItemRefContinued(doc *T, pathItem *PathItem, documentPath *url.URL) (err error) {
 	for _, parameter := range pathItem.Parameters {
 		if err = loader.resolveParameterRef(doc, parameter, documentPath); err != nil {
 			return
