@@ -2,19 +2,18 @@ package openapi3
 
 import (
 	"context"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"sort"
 
 	"github.com/go-openapi/jsonpointer"
-
-	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
 // MediaType is specified by OpenAPI/Swagger 3.0 standard.
-// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#mediaTypeObject
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#media-type-object
 type MediaType struct {
-	ExtensionProps `json:"-" yaml:"-"`
+	Extensions map[string]interface{} `json:"-" yaml:"-"`
 
 	Schema   *SchemaRef           `json:"schema,omitempty" yaml:"schema,omitempty"`
 	Example  interface{}          `json:"example,omitempty" yaml:"example,omitempty"`
@@ -65,17 +64,46 @@ func (mediaType *MediaType) WithEncoding(name string, enc *Encoding) *MediaType
 }
 
 // MarshalJSON returns the JSON encoding of MediaType.
-func (mediaType *MediaType) MarshalJSON() ([]byte, error) {
-	return jsoninfo.MarshalStrictStruct(mediaType)
+func (mediaType MediaType) MarshalJSON() ([]byte, error) {
+	m := make(map[string]interface{}, 4+len(mediaType.Extensions))
+	for k, v := range mediaType.Extensions {
+		m[k] = v
+	}
+	if x := mediaType.Schema; x != nil {
+		m["schema"] = x
+	}
+	if x := mediaType.Example; x != nil {
+		m["example"] = x
+	}
+	if x := mediaType.Examples; len(x) != 0 {
+		m["examples"] = x
+	}
+	if x := mediaType.Encoding; len(x) != 0 {
+		m["encoding"] = x
+	}
+	return json.Marshal(m)
 }
 
 // UnmarshalJSON sets MediaType to a copy of data.
 func (mediaType *MediaType) UnmarshalJSON(data []byte) error {
-	return jsoninfo.UnmarshalStrictStruct(data, mediaType)
+	type MediaTypeBis MediaType
+	var x MediaTypeBis
+	if err := json.Unmarshal(data, &x); err != nil {
+		return err
+	}
+	_ = json.Unmarshal(data, &x.Extensions)
+	delete(x.Extensions, "schema")
+	delete(x.Extensions, "example")
+	delete(x.Extensions, "examples")
+	delete(x.Extensions, "encoding")
+	*mediaType = MediaType(x)
+	return nil
 }
 
 // Validate returns an error if MediaType does not comply with the OpenAPI spec.
-func (mediaType *MediaType) Validate(ctx context.Context) error {
+func (mediaType *MediaType) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	if mediaType == nil {
 		return nil
 	}
@@ -88,35 +116,33 @@ func (mediaType *MediaType) Validate(ctx context.Context) error {
 			return errors.New("example and examples are mutually exclusive")
 		}
 
-		if vo := getValidationOptions(ctx); vo.ExamplesValidationDisabled {
-			return nil
-		}
-
-		if example := mediaType.Example; example != nil {
-			if err := validateExampleValue(ctx, example, schema.Value); err != nil {
-				return fmt.Errorf("invalid example: %w", err)
-			}
-		}
-
-		if examples := mediaType.Examples; examples != nil {
-			names := make([]string, 0, len(examples))
-			for name := range examples {
-				names = append(names, name)
-			}
-			sort.Strings(names)
-			for _, k := range names {
-				v := examples[k]
-				if err := v.Validate(ctx); err != nil {
-					return fmt.Errorf("example %s: %w", k, err)
+		if vo := getValidationOptions(ctx); !vo.examplesValidationDisabled {
+			if example := mediaType.Example; example != nil {
+				if err := validateExampleValue(ctx, example, schema.Value); err != nil {
+					return fmt.Errorf("invalid example: %w", err)
 				}
-				if err := validateExampleValue(ctx, v.Value.Value, schema.Value); err != nil {
-					return fmt.Errorf("example %s: %w", k, err)
+			}
+
+			if examples := mediaType.Examples; examples != nil {
+				names := make([]string, 0, len(examples))
+				for name := range examples {
+					names = append(names, name)
+				}
+				sort.Strings(names)
+				for _, k := range names {
+					v := examples[k]
+					if err := v.Validate(ctx); err != nil {
+						return fmt.Errorf("example %s: %w", k, err)
+					}
+					if err := validateExampleValue(ctx, v.Value.Value, schema.Value); err != nil {
+						return fmt.Errorf("example %s: %w", k, err)
+					}
 				}
 			}
 		}
 	}
 
-	return nil
+	return validateExtensions(ctx, mediaType.Extensions)
 }
 
 // JSONLookup implements github.com/go-openapi/jsonpointer#JSONPointable
@@ -136,6 +162,6 @@ func (mediaType MediaType) JSONLookup(token string) (interface{}, error) {
 	case "encoding":
 		return mediaType.Encoding, nil
 	}
-	v, _, err := jsonpointer.GetForToken(mediaType.ExtensionProps, token)
+	v, _, err := jsonpointer.GetForToken(mediaType.Extensions, token)
 	return v, err
 }
