@@ -2,19 +2,18 @@ package openapi3
 
 import (
 	"context"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"strconv"
 
 	"github.com/go-openapi/jsonpointer"
-
-	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
 // Operation represents "operation" specified by" OpenAPI/Swagger 3.0 standard.
 // See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#operation-object
 type Operation struct {
-	ExtensionProps `json:"-" yaml:"-"`
+	Extensions map[string]interface{} `json:"-" yaml:"-"`
 
 	// Optional tags for documentation.
 	Tags []string `json:"tags,omitempty" yaml:"tags,omitempty"`
@@ -58,13 +57,70 @@ func NewOperation() *Operation {
 }
 
 // MarshalJSON returns the JSON encoding of Operation.
-func (operation *Operation) MarshalJSON() ([]byte, error) {
-	return jsoninfo.MarshalStrictStruct(operation)
+func (operation Operation) MarshalJSON() ([]byte, error) {
+	m := make(map[string]interface{}, 12+len(operation.Extensions))
+	for k, v := range operation.Extensions {
+		m[k] = v
+	}
+	if x := operation.Tags; len(x) != 0 {
+		m["tags"] = x
+	}
+	if x := operation.Summary; x != "" {
+		m["summary"] = x
+	}
+	if x := operation.Description; x != "" {
+		m["description"] = x
+	}
+	if x := operation.OperationID; x != "" {
+		m["operationId"] = x
+	}
+	if x := operation.Parameters; len(x) != 0 {
+		m["parameters"] = x
+	}
+	if x := operation.RequestBody; x != nil {
+		m["requestBody"] = x
+	}
+	m["responses"] = operation.Responses
+	if x := operation.Callbacks; len(x) != 0 {
+		m["callbacks"] = x
+	}
+	if x := operation.Deprecated; x {
+		m["deprecated"] = x
+	}
+	if x := operation.Security; x != nil {
+		m["security"] = x
+	}
+	if x := operation.Servers; x != nil {
+		m["servers"] = x
+	}
+	if x := operation.ExternalDocs; x != nil {
+		m["externalDocs"] = x
+	}
+	return json.Marshal(m)
 }
 
 // UnmarshalJSON sets Operation to a copy of data.
 func (operation *Operation) UnmarshalJSON(data []byte) error {
-	return jsoninfo.UnmarshalStrictStruct(data, operation)
+	type OperationBis Operation
+	var x OperationBis
+	if err := json.Unmarshal(data, &x); err != nil {
+		return err
+	}
+	_ = json.Unmarshal(data, &x.Extensions)
+	delete(x.Extensions, "tags")
+	delete(x.Extensions, "summary")
+	delete(x.Extensions, "description")
+	delete(x.Extensions, "operationId")
+	delete(x.Extensions, "parameters")
+	delete(x.Extensions, "requestBody")
+	delete(x.Extensions, "responses")
+	delete(x.Extensions, "callbacks")
+	delete(x.Extensions, "deprecated")
+	delete(x.Extensions, "security")
+	delete(x.Extensions, "servers")
+	delete(x.Extensions, "externalDocs")
+	*operation = Operation(x)
+	return nil
 }
 
 // JSONLookup implements github.com/go-openapi/jsonpointer#JSONPointable
@@ -101,7 +157,7 @@ func (operation Operation) JSONLookup(token string) (interface{}, error) {
 		return operation.ExternalDocs, nil
 	}
 
-	v, _, err := jsonpointer.GetForToken(operation.ExtensionProps, token)
+	v, _, err := jsonpointer.GetForToken(operation.Extensions, token)
 	return v, err
 }
 
@@ -127,7 +183,9 @@ func (operation *Operation) AddResponse(status int, response *Response) {
 }
 
 // Validate returns an error if Operation does not comply with the OpenAPI spec.
-func (operation *Operation) Validate(ctx context.Context) error {
+func (operation *Operation) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	if v := operation.Parameters; v != nil {
 		if err := v.Validate(ctx); err != nil {
 			return err
@@ -154,5 +212,5 @@ func (operation *Operation) Validate(ctx context.Context) error {
 		}
 	}
 
-	return nil
+	return validateExtensions(ctx, operation.Extensions)
 }
