@@ -801,11 +801,11 @@ var schemaExamples = []schemaExample{
 	{
 		Schema: &Schema{
 			Type: "object",
-			AdditionalProperties: &SchemaRef{
+			AdditionalProperties: AdditionalProperties{Schema: &SchemaRef{
 				Value: &Schema{
 					Type: "number",
 				},
-			},
+			}},
 		},
 		Serialization: map[string]interface{}{
 			"type": "object",
@@ -828,8 +828,8 @@ var schemaExamples = []schemaExample{
 	},
 	{
 		Schema: &Schema{
-			Type:                        "object",
-			AdditionalPropertiesAllowed: BoolPtr(true),
+			Type:                 "object",
+			AdditionalProperties: AdditionalProperties{Has: BoolPtr(true)},
 		},
 		Serialization: map[string]interface{}{
 			"type":                 "object",
@@ -1028,9 +1028,7 @@ func testType(t *testing.T, example schemaTypeExample) func(*testing.T) {
 		}
 		for _, typ := range example.AllInvalid {
 			schema := baseSchema.WithFormat(typ)
-			ctx := WithValidationOptions(context.Background(), &ValidationOptions{
-				SchemaFormatValidationEnabled: true,
-			})
+			ctx := WithValidationOptions(context.Background(), EnableSchemaFormatValidation())
 			err := schema.Validate(ctx)
 			require.Error(t, err)
 		}
@@ -1356,3 +1354,15 @@ enum:
 	err = schema.VisitJSON(map[string]interface{}{"d": "e"})
 	require.Error(t, err)
 }
+
+func TestIssue751(t *testing.T) {
+	schema := &Schema{
+		Type:        "array",
+		UniqueItems: true,
+		Items:       NewStringSchema().NewRef(),
+	}
+	validData := []string{"foo", "bar"}
+	invalidData := []string{"foo", "foo"}
+	require.NoError(t, schema.VisitJSON(validData))
+	require.ErrorContains(t, schema.VisitJSON(invalidData), "duplicate items found")
+}
