@@ -2,21 +2,22 @@ package openapi3
 
 import (
 	"context"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"math"
 	"net/url"
 	"sort"
 	"strings"
-
-	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
 // Servers is specified by OpenAPI/Swagger standard version 3.
 type Servers []*Server
 
 // Validate returns an error if Servers does not comply with the OpenAPI spec.
-func (servers Servers) Validate(ctx context.Context) error {
+func (servers Servers) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	for _, v := range servers {
 		if err := v.Validate(ctx); err != nil {
 			return err
@@ -48,11 +49,11 @@ func (servers Servers) MatchURL(parsedURL *url.URL) (*Server, []string, string)
 }
 
 // Server is specified by OpenAPI/Swagger standard version 3.
-// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#serverObject
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#server-object
 type Server struct {
-	ExtensionProps `json:"-" yaml:"-"`
+	Extensions map[string]interface{} `json:"-" yaml:"-"`
 
-	URL         string                     `json:"url" yaml:"url"`
+	URL         string                     `json:"url" yaml:"url"` // Required
 	Description string                     `json:"description,omitempty" yaml:"description,omitempty"`
 	Variables   map[string]*ServerVariable `json:"variables,omitempty" yaml:"variables,omitempty"`
 }
@@ -82,13 +83,34 @@ func (server *Server) BasePath() (string, error) {
 }
 
 // MarshalJSON returns the JSON encoding of Server.
-func (server *Server) MarshalJSON() ([]byte, error) {
-	return jsoninfo.MarshalStrictStruct(server)
+func (server Server) MarshalJSON() ([]byte, error) {
+	m := make(map[string]interface{}, 3+len(server.Extensions))
+	for k, v := range server.Extensions {
+		m[k] = v
+	}
+	m["url"] = server.URL
+	if x := server.Description; x != "" {
+		m["description"] = x
+	}
+	if x := server.Variables; len(x) != 0 {
+		m["variables"] = x
+	}
+	return json.Marshal(m)
 }
 
 // UnmarshalJSON sets Server to a copy of data.
 func (server *Server) UnmarshalJSON(data []byte) error {
-	return jsoninfo.UnmarshalStrictStruct(data, server)
+	type ServerBis Server
+	var x ServerBis
+	if err := json.Unmarshal(data, &x); err != nil {
+		return err
+	}
+	_ = json.Unmarshal(data, &x.Extensions)
+	delete(x.Extensions, "url")
+	delete(x.Extensions, "description")
+	delete(x.Extensions, "variables")
+	*server = Server(x)
+	return nil
 }
 
 func (server Server) ParameterNames() ([]string, error) {
@@ -163,7 +185,9 @@ func (server Server) MatchRawURL(input string) ([]string, string, bool) {
 }
 
 // Validate returns an error if Server does not comply with the OpenAPI spec.
-func (server *Server) Validate(ctx context.Context) (err error) {
+func (server *Server) Validate(ctx context.Context, opts ...ValidationOption) (err error) {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	if server.URL == "" {
 		return errors.New("value of url must be a non-empty string")
 	}
@@ -191,13 +215,14 @@ func (server *Server) Validate(ctx context.Context) (err error) {
 			return
 		}
 	}
-	return
+
+	return validateExtensions(ctx, server.Extensions)
 }
 
 // ServerVariable is specified by OpenAPI/Swagger standard version 3.
 // See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#server-variable-object
 type ServerVariable struct {
-	ExtensionProps `json:"-" yaml:"-"`
+	Extensions map[string]interface{} `json:"-" yaml:"-"`
 
 	Enum        []string `json:"enum,omitempty" yaml:"enum,omitempty"`
 	Default     string   `json:"default,omitempty" yaml:"default,omitempty"`
@@ -205,17 +230,42 @@ type ServerVariable struct {
 }
 
 // MarshalJSON returns the JSON encoding of ServerVariable.
-func (serverVariable *ServerVariable) MarshalJSON() ([]byte, error) {
-	return jsoninfo.MarshalStrictStruct(serverVariable)
+func (serverVariable ServerVariable) MarshalJSON() ([]byte, error) {
+	m := make(map[string]interface{}, 4+len(serverVariable.Extensions))
+	for k, v := range serverVariable.Extensions {
+		m[k] = v
+	}
+	if x := serverVariable.Enum; len(x) != 0 {
+		m["enum"] = x
+	}
+	if x := serverVariable.Default; x != "" {
+		m["default"] = x
+	}
+	if x := serverVariable.Description; x != "" {
+		m["description"] = x
+	}
+	return json.Marshal(m)
 }
 
 // UnmarshalJSON sets ServerVariable to a copy of data.
 func (serverVariable *ServerVariable) UnmarshalJSON(data []byte) error {
-	return jsoninfo.UnmarshalStrictStruct(data, serverVariable)
+	type ServerVariableBis ServerVariable
+	var x ServerVariableBis
+	if err := json.Unmarshal(data, &x); err != nil {
+		return err
+	}
+	_ = json.Unmarshal(data, &x.Extensions)
+	delete(x.Extensions, "enum")
+	delete(x.Extensions, "default")
+	delete(x.Extensions, "description")
+	*serverVariable = ServerVariable(x)
+	return nil
 }
 
 // Validate returns an error if ServerVariable does not comply with the OpenAPI spec.
-func (serverVariable *ServerVariable) Validate(ctx context.Context) error {
+func (serverVariable *ServerVariable) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	if serverVariable.Default == "" {
 		data, err := serverVariable.MarshalJSON()
 		if err != nil {
@@ -223,5 +273,6 @@ func (serverVariable *ServerVariable) Validate(ctx context.Context) error {
 		}
 		return fmt.Errorf("field default is required in %s", data)
 	}
-	return nil
+
+	return validateExtensions(ctx, serverVariable.Extensions)
 }
