@@ -2,9 +2,8 @@ package openapi3
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
-
-	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
 // Tags is specified by OpenAPI/Swagger 3.0 standard.
@@ -20,7 +19,9 @@ func (tags Tags) Get(name string) *Tag {
 }
 
 // Validate returns an error if Tags does not comply with the OpenAPI spec.
-func (tags Tags) Validate(ctx context.Context) error {
+func (tags Tags) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	for _, v := range tags {
 		if err := v.Validate(ctx); err != nil {
 			return err
@@ -30,9 +31,9 @@ func (tags Tags) Validate(ctx context.Context) error {
 }
 
 // Tag is specified by OpenAPI/Swagger 3.0 standard.
-// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#tagObject
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#tag-object
 type Tag struct {
-	ExtensionProps `json:"-" yaml:"-"`
+	Extensions map[string]interface{} `json:"-" yaml:"-"`
 
 	Name         string        `json:"name,omitempty" yaml:"name,omitempty"`
 	Description  string        `json:"description,omitempty" yaml:"description,omitempty"`
@@ -40,21 +41,47 @@ type Tag struct {
 }
 
 // MarshalJSON returns the JSON encoding of Tag.
-func (t *Tag) MarshalJSON() ([]byte, error) {
-	return jsoninfo.MarshalStrictStruct(t)
+func (t Tag) MarshalJSON() ([]byte, error) {
+	m := make(map[string]interface{}, 3+len(t.Extensions))
+	for k, v := range t.Extensions {
+		m[k] = v
+	}
+	if x := t.Name; x != "" {
+		m["name"] = x
+	}
+	if x := t.Description; x != "" {
+		m["description"] = x
+	}
+	if x := t.ExternalDocs; x != nil {
+		m["externalDocs"] = x
+	}
+	return json.Marshal(m)
 }
 
 // UnmarshalJSON sets Tag to a copy of data.
 func (t *Tag) UnmarshalJSON(data []byte) error {
-	return jsoninfo.UnmarshalStrictStruct(data, t)
+	type TagBis Tag
+	var x TagBis
+	if err := json.Unmarshal(data, &x); err != nil {
+		return err
+	}
+	_ = json.Unmarshal(data, &x.Extensions)
+	delete(x.Extensions, "name")
+	delete(x.Extensions, "description")
+	delete(x.Extensions, "externalDocs")
+	*t = Tag(x)
+	return nil
 }
 
 // Validate returns an error if Tag does not comply with the OpenAPI spec.
-func (t *Tag) Validate(ctx context.Context) error {
+func (t *Tag) Validate(ctx context.Context, opts ...ValidationOption) error {
+	ctx = WithValidationOptions(ctx, opts...)
+
 	if v := t.ExternalDocs; v != nil {
 		if err := v.Validate(ctx); err != nil {
 			return fmt.Errorf("invalid external docs: %w", err)
 		}
 	}
-	return nil
+
+	return validateExtensions(ctx, t.Extensions)
 }
